#!/bin/bash

### example syntax
### srg.sh -t 'minion_id'

### Check for root
if [ $EUID -ne 0 ]; then
    echo "Must be run as root. Please see $0 --help."
    exit 1
fi

### Parse arguments will need to add more later
while [ "$1" ]; do
  case "$1" in
    -t|--target=* )
      # target minion-id i.e. controller-uuid
      if [ "$1" == "-t" ]; then
        shift
        host="$1"
      else
        host=$(echo "$1" | cut -d= -f2-)
      fi
      ;;
    -h|--help )
      # Get that user a help page.
      echo "usage: $0 [-h|--help] [-t | --target=minion-id]"
      echo '   options:'
      echo '   -t minion-id | --target=minion-id   set minion id to use as the target for this script'
      echo 'NOTE: Must be run as root.'
      exit 0
      ;;
  esac
  shift
done

if ! salt "$host" test.ping &>/dev/null; then
  echo 'target is either invalid or offline and is not responding'
  exit 2
fi

info=$(salt "$host" grains.item host ipv4 hwaddr_interfaces id os)
export hostname=$(echo "$info" | grep "host:" -A 1 | tail -n 1 | awk '{ print $1 }')
export ip_address=$(echo "$info" | grep "ipv4:" -A 2 | tail -n 1 | awk '{ print $2 }')
export mac_address=$(echo "$info" | grep "hwaddr_interfaces:" -A 3 | tail -n 1 | awk '{ print $1 }')
export fqdn=$(echo "$info" | grep "id:" -A 1 | tail -n 1 | awk '{ print $1 }')
export os="MySQL"

cp "$os"_SRG_TEMPLATE.ckl "$fqdn".ckl
checklist="$fqdn".ckl
echo "$checklist"

perl -i -0pe 's/(?<=ASSET)(.*?)<HOST_NAME><\/HOST_NAME>/$1<HOST_NAME>$ENV{hostname}<\/HOST_NAME>/s' "$checklist"
perl -i -0pe 's/(?<=ASSET)(.*?)<HOST_IP><\/HOST_IP>/$1<HOST_IP>$ENV{ip_address}<\/HOST_IP>/s' "$checklist"
perl -i -0pe 's/(?<=ASSET)(.*?)<HOST_MAC><\/HOST_MAC>/$1<HOST_MAC>$ENV{mac_address}<\/HOST_MAC>/s' "$checklist"
perl -i -0pe 's/(?<=ASSET)(.*?)<HOST_FQDN><\/HOST_FQDN>/$1<HOST_FQDN>$ENV{fqdn}<\/HOST_FQDN>/s' "$checklist"

grep '<VULN>' -A 3 "$checklist" |                           \
grep 'V-' |                                                 \
grep -oP '(?<=<ATTRIBUTE_DATA>).+?(?=</ATTRIBUTE_DATA>)' |  \
while read vuln; do
### The next line is the important part.  Whatever command you run should take $vuln as input and perform
### different checks based on its value.  In our case, we have a special srg folder with state files named
### $vuln.sls that will be executed.  We are currently building these state files.
### stdout is written to comments, stderr is written to details.

  salt "$fqdn" state.apply "srg-checks/$os/$vuln" 2> stderr 1> stdout
  export vuln
  export comments=$(cat stdout | sed 's/&/&amp;/g')
  export details=$(cat stderr | sed 's/&/&amp;/g')
  perl -i -0pe 's/(?<=$ENV{vuln})(.*?)<COMMENTS><\/COMMENTS>/$1<COMMENTS>$ENV{comments}<\/COMMENTS>/s' "$checklist"
  perl -i -0pe 's/(?<=$ENV{vuln})(.*?)<FINDING_DETAILS><\/FINDING_DETAILS>/$1<FINDING_DETAILS>$ENV{details}<\/FINDING_DETAILS>/s' "$checklist"
  if [ -n "$details" ]; then
    perl -i -0pe 's/(?<=$ENV{vuln})(.*?)<STATUS>Not_Reviewed<\/STATUS>/$1<STATUS>Open<\/STATUS>/s' "$checklist"
  else
    perl -i -0pe 's/(?<=$ENV{vuln})(.*?)<STATUS>Not_Reviewed<\/STATUS>/$1<STATUS>NotAFinding<\/STATUS>/s' "$checklist"
  fi
  rm -f "$stdout"
  rm -f "$stderr"
done
